/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { getElementFromFixture, triggerEvent } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createInvalidParam } from 'libs/tech-shared/test/error';
import { MockNgControl } from '../../../../test/form/MockNgControl';
import { CheckboxEditorComponent } from './checkbox-editor.component';

describe('CheckboxEditorComponent', () => {
  let component: CheckboxEditorComponent;
  let fixture: ComponentFixture<CheckboxEditorComponent>;

  const labelText: string = faker.word.noun();
  const errorMessageId: string = getDataTestIdOf(labelText + '-checkbox-editor-error');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [CheckboxEditorComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(CheckboxEditorComponent);
    component = fixture.componentInstance;
    component.label = labelText;
    component.control = new MockNgControl();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('hasError', () => {
      it('should return true', () => {
        component.fieldControl.setErrors({ 1: createInvalidParam() });
        component._updateInvalidParams();
        const result: boolean = component.hasError;

        expect(result).toBe(true);
      });

      it('should return false', () => {
        component.fieldControl.setErrors(null);

        const result: boolean = component.hasError;

        expect(result).toBe(false);
      });
    });
  });

  describe('template', () => {
    describe('error message', () => {
      it('should show', () => {
        component.fieldControl.setErrors({ 1: createInvalidParam() });
        component._updateInvalidParams();
        fixture.detectChanges();

        const element: HTMLElement = getElementFromFixture(fixture, errorMessageId);

        expect(element).not.toBeEmptyDOMElement();
      });

      it('should hide', () => {
        const element: HTMLElement = getElementFromFixture(fixture, errorMessageId);

        expect(element).toBeEmptyDOMElement();
      });
    });
  });

  describe('input change', () => {
    it('should emit input change', () => {
      component.inputChange.emit = jest.fn();

      triggerEvent({
        fixture,
        elementSelector: 'ods-checkbox',
        name: 'inputChange',
        data: true,
      });

      expect(component.inputChange.emit).toHaveBeenCalledWith(true);
    });
  });
});
